@@ -34,7 +34,7 @@
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
-#include <libempathy/empathy-contact-manager.h>
+#include <libempathy/empathy-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-log-manager.h>
 #include <libempathy/empathy-chatroom-manager.h>
@@ -60,8 +60,6 @@
 
 #define DEBUG_DOMAIN "ChatWindow"
 
-#define URGENCY_TIMEOUT 60*1000
-
 struct _EmpathyChatWindowPriv {
 	EmpathyChatroomManager *chatroom_manager;
 	GList                 *chats;
@@ -73,16 +71,14 @@ struct _EmpathyChatWindowPriv {
 	gboolean               page_added;
 	gboolean               dnd_same_window;
 
-	guint                  urgency_timeout_id;
-
 	GtkWidget             *dialog;
 	GtkWidget             *notebook;
 
-	GtkTooltips           *tooltips;
-
 	/* Menu items. */
 	GtkWidget             *menu_conv_clear;
 	GtkWidget             *menu_conv_insert_smiley;
+	GtkWidget             *menu_conv_call;
+	GtkWidget             *menu_conv_call_separator;
 	GtkWidget             *menu_conv_log;
 	GtkWidget             *menu_conv_separator;
 	GtkWidget             *menu_conv_add_contact;
@@ -138,6 +134,8 @@ static void       chat_window_info_activate_cb          (GtkWidget             *
 							 EmpathyChatWindow      *window);
 static void       chat_window_add_contact_activate_cb   (GtkWidget             *menuitem,
 							 EmpathyChatWindow      *window);
+static void       chat_window_call_activate_cb          (GtkWidget             *menuitem,
+							 EmpathyChatWindow      *window);
 static void       chat_window_log_activate_cb           (GtkWidget             *menuitem,
 							 EmpathyChatWindow      *window);
 static void       chat_window_show_contacts_toggled_cb  (GtkWidget             *menuitem,
@@ -277,8 +275,6 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 
 	priv = GET_PRIV (window);
 
-	priv->tooltips = g_object_ref_sink (gtk_tooltips_new ());
-
 	glade = empathy_glade_get_file ("empathy-chat.glade",
 				       "chat_window",
 				       NULL,
@@ -287,6 +283,8 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "menu_conv", &menu_conv,
 				       "menu_conv_clear", &priv->menu_conv_clear,
 				       "menu_conv_insert_smiley", &priv->menu_conv_insert_smiley,
+				       "menu_conv_call", &priv->menu_conv_call,
+				       "menu_conv_call_separator", &priv->menu_conv_call_separator,
 				       "menu_conv_log", &priv->menu_conv_log,
 				       "menu_conv_separator", &priv->menu_conv_separator,
 				       "menu_conv_add_contact", &priv->menu_conv_add_contact,
@@ -313,6 +311,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "chat_window", "configure-event", chat_window_configure_event_cb,
 			      "menu_conv", "activate", chat_window_conv_activate_cb,
 			      "menu_conv_clear", "activate", chat_window_clear_activate_cb,
+			      "menu_conv_call", "activate", chat_window_call_activate_cb,
 			      "menu_conv_log", "activate", chat_window_log_activate_cb,
 			      "menu_conv_add_contact", "activate", chat_window_add_contact_activate_cb,
 			      "menu_conv_info", "activate", chat_window_info_activate_cb,
@@ -342,7 +341,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				  window);
 
 	priv->notebook = gtk_notebook_new ();
- 	gtk_notebook_set_group_id (GTK_NOTEBOOK (priv->notebook), 1); 
+ 	gtk_notebook_set_group (GTK_NOTEBOOK (priv->notebook), "EmpathyChatWindow"); 
 	gtk_box_pack_start (GTK_BOX (chat_vbox), priv->notebook, TRUE, TRUE, 0);
 	gtk_widget_show (priv->notebook);
 
@@ -374,8 +373,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	/* Set up smiley menu */
 	menu = empathy_chat_view_get_smiley_menu (
 		G_CALLBACK (chat_window_insert_smiley_activate_cb),
-		window,
-		priv->tooltips);
+		window);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->menu_conv_insert_smiley), menu);
 
 	/* Set up signals we can't do with glade since we may need to
@@ -503,13 +501,8 @@ empathy_chat_window_finalize (GObject *object)
 		g_source_remove (priv->save_geometry_id);
 	}
 
-	if (priv->urgency_timeout_id != 0) {
-		g_source_remove (priv->urgency_timeout_id);
-	}
-
 	chat_windows = g_list_remove (chat_windows, window);
 	gtk_widget_destroy (priv->dialog);
-	g_object_unref (priv->tooltips);
 
 	g_signal_handlers_disconnect_by_func (priv->chatroom_manager,
 					      chat_window_update_menu,
@@ -687,61 +680,23 @@ static void
 chat_window_update_title (EmpathyChatWindow *window,
 			  EmpathyChat       *chat)
 {
-	EmpathyChatWindowPriv	*priv;
-	const gchar             *str;
-	gchar			*title;
-	gint  			 n_chats;
+	EmpathyChatWindowPriv *priv;
+	gint                   n_chats;
 	
 	priv = GET_PRIV (window);
 	
 	n_chats = g_list_length (priv->chats);
 	if (n_chats == 1) {
-		if (priv->chats_new_msg) {
-			title = g_strdup_printf (
-				"%s - %s",
-				empathy_chat_get_name (priv->current_chat),
-				_("New Message"));
-		}
-		else if (empathy_chat_is_group_chat (priv->current_chat)) {
-			title = g_strdup_printf (
-				"%s - %s", 
-				empathy_chat_get_name (priv->current_chat),
-				_("Chat Room"));
-		} else {
-			title = g_strdup_printf (
-				"%s - %s", 
-				empathy_chat_get_name (priv->current_chat),
-				_("Conversation"));
-		}
+		gtk_window_set_title (GTK_WINDOW (priv->dialog),
+				      empathy_chat_get_name (priv->current_chat));
 	} else {
-		if (priv->chats_new_msg) {
-			GString *names;
-			GList   *l;
-			gint     n_messages = 0;
+		gchar *title;
 
-			names = g_string_new (NULL);
-
-			for (l = priv->chats_new_msg; l; l = l->next) {
-				n_messages++;
-				g_string_append (names,
-						 empathy_chat_get_name (l->data));
-				if (l->next) {
-					g_string_append (names, ", ");
-				}
-			}
-			
-			str = ngettext ("New Message", "New Messages", n_messages);
-			title = g_strdup_printf ("%s - %s", names->str, str);
-			g_string_free (names, TRUE);
-		} else {
-			str = ngettext ("Conversation", "Conversations (%d)", n_chats);
-			title = g_strdup_printf (str, n_chats);
-		}
+		title = g_strdup_printf (_("Conversations (%d)"), n_chats);
+		gtk_window_set_title (GTK_WINDOW (priv->dialog), title);
+		g_free (title);
 	}
 
-	gtk_window_set_title (GTK_WINDOW (priv->dialog), title);
-	g_free (title);
-
 	if (priv->chats_new_msg) {
 		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog),
 					  EMPATHY_IMAGE_MESSAGE);
@@ -823,17 +778,20 @@ chat_window_update_menu (EmpathyChatWindow *window)
 						   window);
 	} else {
 		EmpathyPrivateChat  *chat;
-		EmpathySubscription  subscription;
 		EmpathyContact      *contact;
+		EmpathyPresence     *presence;
 
 		chat = EMPATHY_PRIVATE_CHAT (priv->current_chat);
 
 		/* Show / Hide widgets */
 		gtk_widget_hide (priv->menu_room);
 
+		/* presence==NULL means this contact refuses to send us his
+		 * presence. By adding the contact we ask the contact to accept
+		 * to send his presence. */
 		contact = empathy_private_chat_get_contact (chat);
-		subscription = empathy_contact_get_subscription (contact);
-		if (!(subscription & EMPATHY_SUBSCRIPTION_FROM)) {
+		presence = empathy_contact_get_presence (contact);
+		if (!presence) {
 			gtk_widget_show (priv->menu_conv_add_contact);
 		} else {
 			gtk_widget_hide (priv->menu_conv_add_contact);
@@ -850,7 +808,7 @@ chat_window_update_menu (EmpathyChatWindow *window)
 }
 
 static void
-chat_window_insert_smiley_activate_cb (GtkWidget        *menuitem,
+chat_window_insert_smiley_activate_cb (GtkWidget         *menuitem,
 				       EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
@@ -896,6 +854,25 @@ chat_window_add_contact_activate_cb (GtkWidget        *menuitem,
 	// FIXME: empathy_add_contact_dialog_show (NULL, contact);
 }
 
+static void
+chat_window_call_activate_cb (GtkWidget         *menuitem,
+			      EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv;
+
+	priv = GET_PRIV (window);
+
+	if (!empathy_chat_is_group_chat (priv->current_chat)) {
+		EmpathyPrivateChat *chat;
+		EmpathyContact     *contact;
+
+		chat = EMPATHY_PRIVATE_CHAT (priv->current_chat);
+		contact = empathy_private_chat_get_contact (chat);
+
+		empathy_call_contact (contact);
+	}
+}
+
 static void
 chat_window_log_activate_cb (GtkWidget        *menuitem,
 			     EmpathyChatWindow *window)
@@ -923,7 +900,7 @@ chat_window_info_activate_cb (GtkWidget        *menuitem,
 
 	empathy_contact_information_dialog_show (contact,
 						 GTK_WINDOW (priv->dialog),
-						 FALSE);
+						 FALSE, FALSE);
 }
 
 static gboolean
@@ -963,20 +940,21 @@ chat_window_configure_event_cb (GtkWidget         *widget,
 	}
 
 	priv->save_geometry_id =
-		g_timeout_add (500,
-			       (GSourceFunc) chat_window_save_geometry_timeout_cb,
-			       window);
+		g_timeout_add_seconds (1,
+				       (GSourceFunc) chat_window_save_geometry_timeout_cb,
+				       window);
 
 	return FALSE;
 }
 
 static void
-chat_window_conv_activate_cb (GtkWidget        *menuitem,
+chat_window_conv_activate_cb (GtkWidget         *menuitem,
 			      EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
-	EmpathyLogManager    *manager;
+	EmpathyLogManager     *manager;
 	gboolean              log_exists = FALSE;
+	gboolean              can_voip = FALSE;
 
 	priv = GET_PRIV (window);
 
@@ -987,7 +965,22 @@ chat_window_conv_activate_cb (GtkWidget        *menuitem,
 						 empathy_chat_is_group_chat (priv->current_chat));
 	g_object_unref (manager);
 
+	if (!empathy_chat_is_group_chat (priv->current_chat)) {
+		EmpathyPrivateChat *chat;
+		EmpathyContact     *contact;
+
+		chat = EMPATHY_PRIVATE_CHAT (priv->current_chat);
+		contact = empathy_private_chat_get_contact (chat);
+		can_voip = empathy_contact_can_voip (contact);
+	}
+
 	gtk_widget_set_sensitive (priv->menu_conv_log, log_exists);
+#ifdef HAVE_VOIP
+	gtk_widget_set_sensitive (priv->menu_conv_call, can_voip);
+#else 
+	g_object_set (priv->menu_conv_call, "visible", FALSE, NULL);
+	g_object_set (priv->menu_conv_call_separator, "visible", FALSE, NULL);
+#endif
 }
 
 static void
@@ -1292,10 +1285,7 @@ chat_window_update_tooltip (EmpathyChatWindow *window,
 	}
 
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-tooltip-widget");
-	gtk_tooltips_set_tip (priv->tooltips,
-			      widget,
-			      str,
-			      NULL);
+	gtk_widget_set_tooltip_text (widget, str);
 
 	g_free (str);
 }
@@ -1583,12 +1573,13 @@ chat_window_drag_data_received (GtkWidget        *widget,
 {
 	/* FIXME: DnD of contact do not seems to work... */
 	if (info == DND_DRAG_TYPE_CONTACT_ID) {
-		EmpathyContactManager *manager;
-		EmpathyContact         *contact;
-		EmpathyChat            *chat;
-		EmpathyChatWindow      *old_window;
+		EmpathyContactFactory *factory;
+		EmpathyContact        *contact = NULL;
+		EmpathyChat           *chat;
+		EmpathyChatWindow     *old_window;
 		McAccount             *account;
 		const gchar           *id = NULL;
+		gchar                **strv;
 
 		if (selection) {
 			id = (const gchar*) selection->data;
@@ -1596,9 +1587,18 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 		empathy_debug (DEBUG_DOMAIN, "DND contact from roster with id:'%s'", id);
 		
-		manager = empathy_contact_manager_new ();
-		contact = empathy_contact_list_find (EMPATHY_CONTACT_LIST (manager), id);
-		g_object_unref (manager);
+		strv = g_strsplit (id, "/", 2);
+		factory = empathy_contact_factory_new ();
+		account = mc_account_lookup (strv[0]);
+		if (account) {
+			contact = empathy_contact_factory_get_from_id (factory,
+								       account,
+								       strv[1]);
+			g_object_unref (account);
+		}
+		g_object_unref (factory);
+		g_object_unref (account);
+		g_strfreev (strv);
 
 		if (!contact) {
 			empathy_debug (DEBUG_DOMAIN, "DND contact from roster not found");
@@ -1607,11 +1607,17 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		
 		account = empathy_contact_get_account (contact);
 		chat = empathy_chat_window_find_chat (account, id);
+		if (chat) {
+			g_object_ref (chat);
+		} else {
+			chat = EMPATHY_CHAT (empathy_private_chat_new_with_contact (contact));
+		}
 		old_window = empathy_chat_get_window (chat);
 		
 		if (old_window) {
 			if (old_window == window) {
 				gtk_drag_finish (context, TRUE, FALSE, time);
+				g_object_unref (chat);
 				return;
 			}
 			
@@ -1629,6 +1635,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		 * anyway with add_chat() and remove_chat().
 		 */
 		gtk_drag_finish (context, TRUE, FALSE, time);
+		g_object_unref (chat);
 	}
 	else if (info == DND_DRAG_TYPE_TAB) {
 		EmpathyChat        *chat = NULL;
@@ -1673,21 +1680,6 @@ chat_window_drag_data_received (GtkWidget        *widget,
 	}
 }
 
-static gboolean
-chat_window_urgency_timeout_func (EmpathyChatWindow *window)
-{
-	EmpathyChatWindowPriv *priv;
-
-	priv = GET_PRIV (window);
-
-	empathy_debug (DEBUG_DOMAIN, "Turning off urgency hint");
-	gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), FALSE);
-
-	priv->urgency_timeout_id = 0;
-
-	return FALSE;
-}
-
 static void
 chat_window_set_urgency_hint (EmpathyChatWindow *window,
 			      gboolean          urgent)
@@ -1696,29 +1688,9 @@ chat_window_set_urgency_hint (EmpathyChatWindow *window,
 
 	priv = GET_PRIV (window);
 
-	if (!urgent) {
-		/* Remove any existing hint and timeout. */
-		if (priv->urgency_timeout_id) {
-			empathy_debug (DEBUG_DOMAIN, "Turning off urgency hint");
-			gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), FALSE);
-			g_source_remove (priv->urgency_timeout_id);
-			priv->urgency_timeout_id = 0;
-		}
-		return;
-	}
-
-	/* Add a new hint and renew any exising timeout or add a new one. */
-	if (priv->urgency_timeout_id) {
-		g_source_remove (priv->urgency_timeout_id);
-	} else {
-		empathy_debug (DEBUG_DOMAIN, "Turning on urgency hint");
-		gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), TRUE);
-	}
-
-	priv->urgency_timeout_id = g_timeout_add (
-		URGENCY_TIMEOUT,
-		(GSourceFunc) chat_window_urgency_timeout_func,
-		window);
+	empathy_debug (DEBUG_DOMAIN, "Turning %s urgency hint",
+		       urgent ? "on" : "off");
+	gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), urgent);
 }
 
 EmpathyChatWindow *
@@ -1741,12 +1713,15 @@ empathy_chat_window_get_dialog (EmpathyChatWindow *window)
 
 void
 empathy_chat_window_add_chat (EmpathyChatWindow *window,
-			     EmpathyChat	      *chat)
+			      EmpathyChat	*chat)
 {
 	EmpathyChatWindowPriv *priv;
 	GtkWidget            *label;
 	GtkWidget            *child;
 
+	g_return_if_fail (window != NULL);
+	g_return_if_fail (EMPATHY_IS_CHAT (chat));
+
 	priv = GET_PRIV (window);
 
 	/* Reference the chat object */
@@ -1791,11 +1766,14 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 
 void
 empathy_chat_window_remove_chat (EmpathyChatWindow *window,
-				EmpathyChat	 *chat)
+				 EmpathyChat	   *chat)
 {
 	EmpathyChatWindowPriv *priv;
 	gint                  position;
 
+	g_return_if_fail (window != NULL);
+	g_return_if_fail (EMPATHY_IS_CHAT (chat));
+
 	priv = GET_PRIV (window);
 
 	position = gtk_notebook_page_num (GTK_NOTEBOOK (priv->notebook),
@@ -1811,8 +1789,8 @@ empathy_chat_window_remove_chat (EmpathyChatWindow *window,
 
 void
 empathy_chat_window_move_chat (EmpathyChatWindow *old_window,
-			      EmpathyChatWindow *new_window,
-			      EmpathyChat       *chat)
+			       EmpathyChatWindow *new_window,
+			       EmpathyChat       *chat)
 {
 	GtkWidget *widget;
 
@@ -1842,11 +1820,14 @@ empathy_chat_window_move_chat (EmpathyChatWindow *old_window,
 
 void
 empathy_chat_window_switch_to_chat (EmpathyChatWindow *window,
-				   EmpathyChat	    *chat)
+				    EmpathyChat	      *chat)
 {
 	EmpathyChatWindowPriv *priv;
 	gint                  page_num;
 
+	g_return_if_fail (window != NULL);
+	g_return_if_fail (EMPATHY_IS_CHAT (chat));
+
 	priv = GET_PRIV (window);
 
 	page_num = gtk_notebook_page_num (GTK_NOTEBOOK (priv->notebook),
@@ -1872,10 +1853,13 @@ empathy_chat_window_has_focus (EmpathyChatWindow *window)
 
 EmpathyChat *
 empathy_chat_window_find_chat (McAccount   *account,
-			      const gchar *id)
+			       const gchar *id)
 {
 	GList *l;
 
+	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
+	g_return_val_if_fail (!G_STR_EMPTY (id), NULL);
+
 	for (l = chat_windows; l; l = l->next) {
 		EmpathyChatWindowPriv *priv;
 		EmpathyChatWindow     *window;
