@@ -63,11 +63,9 @@
 #define COMPOSING_STOP_TIMEOUT 5
 
 struct _EmpathyChatPriv {
-	EmpathyContactManager *manager;
 	EmpathyLogManager     *log_manager;
 	EmpathyTpChat         *tp_chat;
 	EmpathyChatWindow      *window;
-	GtkTooltips           *tooltips;
 	guint                  composing_stop_timeout_id;
 	gboolean               sensitive;
 	gchar                 *id;
@@ -76,7 +74,7 @@ struct _EmpathyChatPriv {
 	GList                 *compositors;
 	guint                  scroll_idle_id;
 	gboolean               first_tp_chat;
-	EmpathyTime            last_log_timestamp;
+	time_t                 last_log_timestamp;
 	/* Used to automatically shrink a window that has temporarily
 	 * grown due to long input. 
 	 */
@@ -235,9 +233,7 @@ empathy_chat_init (EmpathyChat *chat)
 
 	priv = GET_PRIV (chat);
 
-	priv->manager = empathy_contact_manager_new ();
 	priv->log_manager = empathy_log_manager_new ();
-	priv->tooltips = g_object_ref_sink (gtk_tooltips_new ());
 	priv->default_window_height = -1;
 	priv->vscroll_visible = FALSE;
 	priv->sensitive = TRUE;
@@ -301,9 +297,7 @@ chat_finalize (GObject *object)
 
 	chat_composing_remove_timeout (chat);
 	g_object_unref (chat->account);
-	g_object_unref (priv->manager);
 	g_object_unref (priv->log_manager);
-	g_object_unref (priv->tooltips);
 
 	if (priv->tp_chat) {
 		g_object_unref (priv->tp_chat);
@@ -402,7 +396,7 @@ chat_message_received_cb (EmpathyTpChat  *tp_chat,
 {
 	EmpathyChatPriv *priv;
 	EmpathyContact  *sender;
-	EmpathyTime      timestamp;
+	time_t           timestamp;
 
 	priv = GET_PRIV (chat);
 
@@ -433,6 +427,11 @@ chat_message_received_cb (EmpathyTpChat  *tp_chat,
 		// FIXME: empathy_sound_play (EMPATHY_SOUND_CHAT);
 	}
 
+	/* We received a message so the contact is no more composing */
+	chat_state_changed_cb (tp_chat, sender,
+			       TP_CHANNEL_CHAT_STATE_ACTIVE,
+			       chat);
+
 	g_signal_emit (chat, signals[NEW_MESSAGE], 0, message, FALSE);
 }
 
@@ -927,8 +926,7 @@ chat_text_populate_popup_cb (GtkTextView *view,
 
 	smiley_menu = empathy_chat_view_get_smiley_menu (
 		G_CALLBACK (chat_insert_smiley_activate_cb),
-		chat,
-		priv->tooltips);
+		chat);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), smiley_menu);
 
 	/* Add the spell check menu item. */
@@ -1029,8 +1027,8 @@ chat_composing_start (EmpathyChat *chat)
 					   TP_CHANNEL_CHAT_STATE_COMPOSING);
 	}
 
-	priv->composing_stop_timeout_id = g_timeout_add (
-		1000 * COMPOSING_STOP_TIMEOUT,
+	priv->composing_stop_timeout_id = g_timeout_add_seconds (
+		COMPOSING_STOP_TIMEOUT,
 		(GSourceFunc) chat_composing_stop_timeout_cb,
 		chat);
 }
