@@ -230,7 +230,7 @@ static const GtkActionEntry entries[] = {
 	  N_("_Send File..."), NULL, N_("Send a file"),
 	  G_CALLBACK (contact_list_view_action_cb)
 	},
-	{ "Log", GTK_STOCK_JUSTIFY_LEFT,
+	{ "Log", EMPATHY_IMAGE_LOG,
 	  N_("_View Previous Conversations"), NULL, N_("View previous conversations with this contact"),
 	  G_CALLBACK (contact_list_view_action_cb)
 	},
@@ -500,7 +500,9 @@ empathy_contact_list_view_get_selected (EmpathyContactListView *view)
 		return NULL;
 	}
 
-	gtk_tree_model_get (model, &iter, COL_CONTACT, &contact, -1);
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
 
 	return contact;
 }
@@ -525,8 +527,8 @@ empathy_contact_list_view_get_selected_group (EmpathyContactListView *view)
 	}
 
 	gtk_tree_model_get (model, &iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_NAME, &name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
 			    -1);
 
 	if (!is_group) {
@@ -631,11 +633,11 @@ contact_list_view_setup (EmpathyContactListView *view)
 		view, NULL);
 
 	gtk_tree_view_column_add_attribute (col, cell,
-					    "name", COL_NAME);
+					    "name", EMPATHY_CONTACT_LIST_STORE_COL_NAME);
 	gtk_tree_view_column_add_attribute (col, cell,
-					    "status", COL_STATUS);
+					    "status", EMPATHY_CONTACT_LIST_STORE_COL_STATUS);
 	gtk_tree_view_column_add_attribute (col, cell,
-					    "is_group", COL_IS_GROUP);
+					    "is_group", EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP);
 
 #ifdef HAVE_VOIP
 	/* Voip Capability Icon */
@@ -719,8 +721,8 @@ contact_list_view_row_has_child_toggled_cb (GtkTreeModel          *model,
 	gchar    *name = NULL;
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_NAME, &name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
 			    -1);
 
 	if (!is_group || G_STR_EMPTY (name)) {
@@ -891,8 +893,7 @@ contact_list_view_drag_motion (GtkWidget      *widget,
 		dm->view = EMPATHY_CONTACT_LIST_VIEW (widget);
 		dm->path = gtk_tree_path_copy (path);
 
-		dm->timeout_id = g_timeout_add (
-			1500,
+		dm->timeout_id = g_timeout_add_seconds (1,
 			(GSourceFunc) contact_list_view_drag_motion_cb,
 			dm);
 	}
@@ -1073,9 +1074,9 @@ contact_list_view_pixbuf_cell_data_func (GtkTreeViewColumn     *tree_column,
 	gboolean  is_active;
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_IS_ACTIVE, &is_active,
-			    COL_ICON_STATUS, &icon_name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
+			    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, &icon_name,
 			    -1);
 
 	g_object_set (cell,
@@ -1101,9 +1102,9 @@ contact_list_view_voip_cell_data_func (GtkTreeViewColumn      *tree_column,
 	gboolean can_voip;
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_IS_ACTIVE, &is_active,
-			    COL_CAN_VOIP, &can_voip,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VOIP, &can_voip,
 			    -1);
 
 	g_object_set (cell,
@@ -1128,10 +1129,10 @@ contact_list_view_avatar_cell_data_func (GtkTreeViewColumn     *tree_column,
 	gboolean   is_active;
 
 	gtk_tree_model_get (model, iter,
-			    COL_PIXBUF_AVATAR, &pixbuf,
-			    COL_PIXBUF_AVATAR_VISIBLE, &show_avatar,
-			    COL_IS_GROUP, &is_group,
-			    COL_IS_ACTIVE, &is_active,
+			    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR, &pixbuf,
+			    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, &show_avatar,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
 			    -1);
 
 	g_object_set (cell,
@@ -1158,9 +1159,9 @@ contact_list_view_text_cell_data_func (GtkTreeViewColumn     *tree_column,
 	gboolean show_status;
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_IS_ACTIVE, &is_active,
-			    COL_STATUS_VISIBLE, &show_status,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
+			    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, &show_status,
 			    -1);
 
 	g_object_set (cell,
@@ -1181,8 +1182,8 @@ contact_list_view_expander_cell_data_func (GtkTreeViewColumn     *column,
 	gboolean is_active;
 
 	gtk_tree_model_get (model, iter,
-			    COL_IS_GROUP, &is_group,
-			    COL_IS_ACTIVE, &is_active,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, &is_active,
 			    -1);
 
 	if (gtk_tree_model_iter_has_child (model, iter)) {
@@ -1220,8 +1221,10 @@ contact_list_view_get_contact_menu (EmpathyContactListView *view,
 	action = gtk_ui_manager_get_action (priv->ui, "/Contact/Log");
 	gtk_action_set_sensitive (action, can_show_log);
 
+#ifdef HAVE_VOIP
 	action = gtk_ui_manager_get_action (priv->ui, "/Contact/Call");
 	gtk_action_set_sensitive (action, can_voip);
+#endif
 
 	action = gtk_ui_manager_get_action (priv->ui, "/Contact/SendFile");
 	gtk_action_set_visible (action, can_send_file);
@@ -1270,7 +1273,9 @@ contact_list_view_button_press_event_cb (EmpathyContactListView *view,
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_path_free (path);
 
-	gtk_tree_model_get (model, &iter, COL_CONTACT, &contact, -1);
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
 
 	if (contact) {
 		menu = empathy_contact_list_view_get_contact_menu (view, contact);
@@ -1310,7 +1315,9 @@ contact_list_view_row_activated_cb (EmpathyContactListView *view,
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
 
 	gtk_tree_model_get_iter (model, &iter, path);
-	gtk_tree_model_get (model, &iter, COL_CONTACT, &contact, -1);
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
 
 	if (contact) {
 		contact_list_view_action_activated (view, contact);
@@ -1338,7 +1345,9 @@ contact_list_view_voip_activated_cb (EmpathyCellRendererActivatable *cell,
 		return;
 	}
 
-	gtk_tree_model_get (model, &iter, COL_CONTACT, &contact, -1);
+	gtk_tree_model_get (model, &iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    -1);
 
 	if (contact) {
 		contact_list_view_voip_activated (view, contact);
@@ -1361,7 +1370,7 @@ contact_list_view_row_expand_or_collapse_cb (EmpathyContactListView *view,
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
 
 	gtk_tree_model_get (model, iter,
-			    COL_NAME, &name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
 			    -1);
 
 	expanded = GPOINTER_TO_INT (user_data);
@@ -1400,10 +1409,10 @@ contact_list_view_action_cb (GtkAction             *action,
 		contact_list_view_voip_activated (view, contact);
 	}
 	else if (contact && strcmp (name, "Information") == 0) {
-		empathy_contact_information_dialog_show (contact, parent, FALSE);
+		empathy_contact_information_dialog_show (contact, parent, FALSE, FALSE);
 	}
 	else if (contact && strcmp (name, "Edit") == 0) {
-		empathy_contact_information_dialog_show (contact, parent, TRUE);
+		empathy_contact_information_dialog_show (contact, parent, TRUE, FALSE);
 	}
 	else if (contact && strcmp (name, "Remove") == 0) {
 		/* FIXME: Ask for confirmation */
